// An API key is required in order to execute the following tests. The key can be generated
// from the Google API Console (https://console.developers.google.com/apis/credentials) and
// can be directly pasted in this file (var gmApiKey='####...'). Alternatively you can specify
// the key as part of the unit test URL query, eg: http://.../test/local/?gmApiKey=#####
(function() {
    var apiKey = 0; // TODO jasmine._options.gmApiKey;

    Ext.merge(Ext.manifest, {
        mashup: {
            map: {
                key: apiKey
            }
        }
    });

    return apiKey ? topSuite : xtopSuite;
}())('Ext.ux.google.Map', function() {
    var views = {};

    function acquireView(config) {
        var view = Ext.create(Ext.merge({
            xtype: 'google-map',
            renderTo: Ext.getBody(),
            height: 256,
            width: 512
        }, config || {}));

        views[view.getId()] = view;

        return view;
    }

    // Calls the given function when the map become idle, can be used to perform
    // tests after zooming/panning or prevent DOM leaks during test teardown.
    function onMapIdle(map, callback) {
        google.maps.event.addListenerOnce(map, 'idle', callback);
    }

    function getGoogleMarkers(view) {
        return view.getMarkers().getRange().map(function(record) {
            var marker = view.getMarkerForRecord(record);

            expect(marker).toBeDefined();
            expect(marker.getMap()).toBe(view.getMap());

            return marker;
        });
    }

    afterEach(function() {
        Ext.Object.getValues(views).forEach(function(view) {
            var id = view.getId();

            delete views[id];
            view.destroy();
        });

        // Prevent Google Maps __e3_ global variables resource leaks
        // https://code.google.com/p/gmaps-api-issues/issues/detail?id=9108
        delete __e3_;
    });

    describe('markers', function() {

        // Note(SB): can't find any way to query the existing markers in the google.Map object,
        // so let's check that the expando marker is valid and registered for the tested map.

        it('should load markers from a single object', function() {
            var view = acquireView({
                    markers: {
                        position: {
                            lat: 52.3745291,
                            lng: 4.7585319
                        }
                    }
                });

            expect(view.getMarkers().isStore).toBeTruthy();
            expect(view.getMarkers().getCount()).toBe(1);
            expect(getGoogleMarkers(view).length).toBe(1);
        });

        it('should load markers from a single record', function() {
            var view = acquireView({
                    markers: Ext.create(Ext.data.Model, {
                        position: {
                            lat: 52.3745291,
                            lng: 4.7585319
                        }
                    })
                });

            expect(view.getMarkers().isStore).toBeTruthy();
            expect(view.getMarkers().getCount()).toBe(1);
            expect(getGoogleMarkers(view).length).toBe(1);
        });

        it('should load markers from a store', function() {
            var store = Ext.create('Ext.data.Store', {
                    data: [{
                        position: {
                            lat: 52.3745291,
                            lng: 4.7585319
                        }
                    }, {
                        position: {
                            lat: 35.6732619,
                            lng: 139.5703047
                        }
                    }]
                }),
                view = acquireView({
                    markers: store
                });

            expect(view.getMarkers().isStore).toBeTruthy();
            expect(view.getMarkers().getCount()).toBe(2);
            expect(getGoogleMarkers(view).length).toBe(2);
        });

        it('should sync markers when adding, removing or updating the store', function() {
            var store = Ext.create('Ext.data.Store', {
                    data: [{
                        title: 'Amsterdam',
                        position: {
                            lat: 52.3745291,
                            lng: 4.7585319
                        }
                    }]
                }),
                view = acquireView({
                    markers: store
                }),
                store = view.getMarkers(),
                record, marker;

            expect(store.isStore).toBeTruthy();
            expect(store.getCount()).toBe(1);
            expect(getGoogleMarkers(view).length).toBe(1);

            store.add({
                title: 'Tokyo',
                position: {
                    lat: 35.6732619,
                    lng: 139.5703047
                }
            }, {
                title: 'Paris',
                position: {
                    lat: 48.8588377,
                    lng: 2.2775175
                }
            });

            expect(store.getCount()).toBe(3);
            expect(getGoogleMarkers(view).length).toBe(3);

            record = store.getAt(2);
            marker = view.getMarkerForRecord(record);
            expect(marker.getMap()).toBe(view.getMap());
            expect(marker.getTitle()).toBe('Paris');
            expect(marker.getPosition().lat()).toBeCloseTo(48.8588377, 6);
            expect(marker.getPosition().lng()).toBeCloseTo(2.2775175, 6);

            record.set({
                title: 'New York City',
                position: {
                    lat: 40.705311,
                    lng: -74.258183
                }
            });

            expect(marker.getTitle()).toBe('New York City');
            expect(marker.getPosition().lat()).toBeCloseTo(40.705311, 6);
            expect(marker.getPosition().lng()).toBeCloseTo(-74.258183, 6);

            store.remove(record);

            expect(store.getCount()).toBe(2);
            expect(getGoogleMarkers(view).length).toBe(2);
            expect(view.getMarkerForRecord(record)).toBeNull();
            expect(marker.getMap()).toBeNull();
        });

        it('should correctly build markers from template', function() {
            var view = acquireView({
                    markers: {
                        address: {
                            city: 'Amsterdam',
                            geo: {
                                latitude: 52.3745291,
                                longitude: 4.7585319
                            }
                        }
                    },
                    markerTemplate: {
                        title: '{address.city}',
                        position: {
                            lat: '{address.geo.latitude}',
                            lng: '{address.geo.longitude}'
                        }
                    }
                }),
                markers = getGoogleMarkers(view);

            expect(markers.length).toBe(1);
            expect(markers[0].getTitle()).toBe('Amsterdam');
            expect(markers[0].getPosition().lat()).toBeCloseTo(52.3745291, 6);
            expect(markers[0].getPosition().lng()).toBeCloseTo(4.7585319, 6);
        });

        it('should support "DROP" and "BOUNCE" as animation value', function() {
            var view = acquireView({
                    markers: {
                        animation: 'DROP',
                        position: {
                            lat: 52.3745291,
                            lng: 4.7585319
                        }
                    }
                }),
                record = view.getMarkers().getAt(0),
                marker = getGoogleMarkers(view)[0];

            expect(record.get('animation')).toBe('DROP');
            expect(marker.getAnimation()).toBe(google.maps.Animation.DROP);

            record.set('animation', 'FOOBAR');
            expect(record.get('animation')).toBe('FOOBAR');
            expect(marker.getAnimation()).toBe(null);

            record.set('animation', 'BOUNCE');
            expect(record.get('animation')).toBe('BOUNCE');
            expect(marker.getAnimation()).toBe(google.maps.Animation.BOUNCE);

            record.set('animation', null);
            expect(record.get('animation')).toBeNull();
            expect(marker.getAnimation()).toBeNull();
        });

        it('should fire marker events', function() {
            var latLng = new google.maps.LatLng(52.3745291, 4.7585319),
                spy = jasmine.createSpy(),
                view = acquireView({
                    markers: [{
                        title: 'New York City',
                        position: {
                            lat: 40.705311,
                            lng: -74.258183
                        }
                    }, {
                        title: 'Amsterdam',
                        position: latLng
                    }]
                }),
                markers = getGoogleMarkers(view),
                types = view.markerEvents,
                store = view.getMarkers();

            types.forEach(function(type) {
                view.on('marker' + type, spy);
            });

            types.forEach(function(type) {
                google.maps.event.trigger(markers[1], type, {
                    latLng: latLng
                });
            });

            expect(types.length).toBe(10);  // sanity check
            expect(spy.callCount).toBe(types.length);

            types.forEach(function(type, index) {
                var info = spy.calls[index].args[1];

                expect(spy.calls[index].args[0]).toBe(view);
                expect(info.index).toBe(1);
                expect(info.record).toBe(store.getAt(1));
                expect(info.marker).toBe(markers[1]);
                expect(info.marker instanceof google.maps.Marker).toBeTruthy();
                expect(info.event).toEqual({ latLng: latLng });
            });
        });
    });

    describe("map center", function() {
        // https://sencha.jira.com/browse/EXT-300
        it('should be centered when initially hidden', function(done) {
            var view = acquireView({
                    hidden: true,
                    mapOptions: {
                        center: {
                            latitude: -13.1631412,
                            longitude: -72.5471516
                        }
                    }
                });

            var map = view.getMap();

            google.maps.event.addListenerOnce(map, 'bounds_changed', function() {
                view.show();
                view.on({
                    painted: function() {
                        onMapIdle(map, function() {
                            var center = map.getCenter();

                            expect(center.lat()).toBeCloseTo(-13.1631412, 8);
                            expect(center.lng()).toBeCloseTo(-72.5471516, 8);
                            Ext.defer(done, 1000);
                        });
                    }
                });
            });
        });
    });
});
